ifndef custres_wdl;
define custres_wdl;
// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 1.2
// engineReq: 6.4
// date: 061011
//
// title: Custom resources handling routines.
// title: 2006,07 by Leonid Tsinitskiy (LionTs).
// text: See CustRes.chm for details.
// class: LIONTS
// type: USER
// image: custres.pcx
// help: Handle the custom resources
//
// needs: 
//
// external:
//
// prefix: CR_
//
// ENDHEADER
// ---------------------------------------------------------------------

/*
**	2006,07 by Leonid Tsinitskiy (LionTs)
**		Custom resources handling routines
**			- place CUSTRES.DLL into the 3DGS plugins directory (or into the game folder)
**			- include this file in your main script file
**	
**		look at CustRes.chm for details
**		http://www.Lion-Ts.com
*/

bind <custres.dll>;	//include custres.dll in a published game
// section: CR Package
// entry_help: Enable this and enter valid filename below if you have
// entry_help:  one CR Package only.
// enable: Use ONE CR Package only
//define UseFileNameStr;
// entry: Name for the custom resource file.
// entry_help: Make your package and enter its filename here.
// entry_help: To manage more than one package you have to edit
// entry_help: C-script manually.
string CR_FileName_str = "";
// section: Security
// entry_help: Enable this and enter password for your CR packages below
// entry_help: to work with ALL CR Packages providing ONE password.
// enable: Use ONE password
//define UsePasswordStr;
// entry: Password for the custom resource file.
// entry_help: Provide password to open your resources.
// entry_help: To provide different passwords you have to edit
// entry_help: C-script manually.
string CR_Password_str = "";
// section: Error handling
// entry_help: Show errors to user with the ERROR() function.
// enable: Show errors
define ShowErrorToScreen;
// entry_help: Print errors into the Acklog.txt, this option applies to
// entry_help: C-script functions only. Custom Resource engine
// entry_help: writes diagnostic messages to log anyway.
// enable: Write errors to ACKLOG.TXT
define PrintErrorToAcklog;
// entry_help: Always exit on errors while processing Custom Resources or
// entry_help: check AbortOnError parameter, passed to
// entry_help: CR_... functions
// enable: Always abort on errors
define AbortOnErrorInGSR;

IFDEF UseFileNameStr;
IFDEF UsePasswordStr;
define CR_QuickMode;
ENDIF;
ENDIF;

/*
**	All functions returns errorcode
**	You may consult with it to provide readable error messages for user
*/
define CR_OK,			0;	// OK - NO ERROR
define CR_ERR_NONE,		0;	// SAME AS ABOVE
define CR_ERR_EOLF,		-100;	// END_OF_LIST_OF_FILE (RESOURCE DIRECTORY)
define CR_ERR_PARAM,	-102;	// PARAMETER ERROR (CHECK PARAMS)
define CR_ERR_BADFILE,	-103;	// BAD RESOURCE (CORRUPTED FILE) 
define CR_ERR_INTERNAL, -104;	// INTERNAL ERROR
define CR_ERR_CRC,		-105;	// CRC ERROR (CORRUPTED FILE)
define CR_ERR_ERRNO,	-1;	// ERROR OCCURRED IN THE FILE SYSTEM, NOT IN THE DLL (CAN'T WRITE FILE)
define CR_ERR_STREAM,	-2;	// STREAM ERROR
define CR_ERR_DATA,		-3;	// DATA ERROR (WRONG PASSWORD OR CORRUPTED FILE)
define CR_ERR_MEM,		-4;	// MEMORY ALLOCATION ERROR
define CR_ERR_BUF,		-5;	// NON FATAL ERROR WITH STREAM
define CR_ERR_VERSION,	-6;	// VERSION ERROR (RESOURCE CREATED WITH OTHER VERSION OF DLL)

// section: Localizable error messages
// entry: CR_STR_ERR_EOLF
// entry_help: File not found in the resource
string CR_STR_ERR_EOLF		=	"File was not found in resource";
// entry: CR_STR_ERR_PARAM
// entry_help: Wrong parameters passed to CUSTRES.DLL
string CR_STR_ERR_PARAM		=	"Wrong parameters passed to CUSTRES.DLL";
// entry: CR_STR_ERR_BADFILE
// entry_help: Bad custom resource or file does not exists
string CR_STR_ERR_BADFILE	=	"Bad custom resource or file does not exists";
// entry: CR_STR_ERR_INTERNAL
// entry_help: Internal error in CUSTRES.DLL
string CR_STR_ERR_INTERNAL	=	"Internal error in CUSTRES.DLL";
// entry: CR_STR_ERR_CRC
// entry_help: Bad checksum of custom resource, corrupted file
string CR_STR_ERR_CRC		=	"Bad checksum of custom resource, corrupted file";
// entry: CR_STR_ERR_ERRNO
// entry_help: Error occurred in the file system, check your disk
string CR_STR_ERR_ERRNO		=	"Error occurred in the file system, check your disk";
// entry: CR_STR_ERR_STREAM
// entry_help: Internal error in CUSTRES.DLL
string CR_STR_ERR_STREAM	=	"Internal error in CUSTRES.DLL";
// entry: CR_STR_ERR_DATA
// entry_help: Bad custom resource, corrupted file or wrong password
string CR_STR_ERR_DATA		=	"Bad custom resource, corrupted file or wrong password";
// entry: CR_STR_ERR_MEM
// entry_help: Not enough memory for custom resource operations
string CR_STR_ERR_MEM		=	"Not enough memory for custom resource operations";
// entry: CR_STR_ERR_BUF
// entry_help: Internal error in CUSTRES.DLL
string CR_STR_ERR_BUF		=	"Internal error in CUSTRES.DLL";
// entry: CR_STR_ERR_VERSION
// entry_help: Wrong version of custom resource
string CR_STR_ERR_VERSION	=	"Wrong version of custom resource";
// entry: CR_STR_ERR_UNKNOWN
// entry_help: Unknown error in custom resource
string CR_STR_ERR_UNKNOWN	=	"Unknown error in custom resource";
// entry: CR_STR_ERR_RESPREF
// entry_help: , Resource: 
string CR_STR_ERR_RESPREF	=	", Resource: ";
// entry: CR_STR_ERR_ABORT
// entry_help: Application aborted because of errors in custom resources.
string CR_STR_ERR_ABORT		=	"Application aborted because of errors in custom resources.";

/*
**	Directly imported from CUSTRES.DLL functions
**	prefixed with "CR_", ends with "_"
**	return VAR errorcode
*/

/*
**	Utility functions
*/

/*
**	Core functions
*/
//open custom resource FILENAME with PASSWORD
dllfunction CR_open_(str_filename, str_password);
/*
**	You have to 'open' resource with cr_open(...) before any call to functions, listed below
*/
//close custom resource file (removes all added buffers for the FILENAME)
dllfunction CR_close_(str_filename);
/*
**	Functions to add files to the engine filesystem
*/
//add all resources to engine filesystem from the FILENAME
dllfunction CR_addall_(str_filename);
//add FILELIST resources to engine filesystem from the FILENAME
dllfunction CR_add_(str_filename, txt_filelist);
//add one resource item to engine filesystem from the FILENAME
dllfunction CR_addone_(str_filename, str_itemname);
//add resource number FILENUMBER to the engine filesystem from the FILENAME
dllfunction CR_addn_(str_filename, var_itemnumber);
/*
**	Functions to remove files from the engine filesystem
*/
//del all resources from engine filesystem presented in the FILENAME
dllfunction CR_delall_(str_filename);
//del FILELIST resources from the engine filesystem presented in the FILENAME
dllfunction CR_del_(str_filename, txt_filelist);
//del one resource from the engine filesystem presented in the FILENAME
dllfunction CR_delone_(str_filename, str_itemname);
//del resource number FILENUMBER from the engine filesystem presented in the FILENAME
dllfunction CR_deln_(str_filename, var_itemnumber);
/*
**	Functions to extract files from the resource
*/
//extract all resources from the FILENAME to the specified PATH
dllfunction CR_extractall_(str_filename, str_path);
//extract FILELIST resources from the FILENAME to the specified PATH
dllfunction CR_extract_(str_filename, str_path, txt_filelist);
//extract one resource from the FILENAME to the specified PATH
dllfunction CR_extractone_(str_filename, str_path, str_itemname);
//extract resource number FILENUMBER from the FILENAME to the specified PATH
dllfunction CR_extractn_(str_filename, str_path, var_itemnumber);
/*
**	Functions to obtain information about a resource
*/
//get number of files
dllfunction CR_getfcount_(str_filename);
//get list of files, the FILELIST size must be equal or larger than files count
dllfunction CR_getflist_(str_filename, txt_filelist);
//is ITEM exists in the FILENAME
dllfunction CR_itemexists_(str_filename, str_itemname);

/*
**	strings for internal usage
*/
string CR_ERROR_STR;	//temp error string
//these strings needed to handle case of predefined filename and filepass
string CR_TEMP_FNAME;
string CR_TEMP_PASS;

/*
**	Helper function
**	show error message if any and abort engine optionaly
*/
function CR_ShowErr(Err, FileName, AbortOnError)
{
	if (err < CR_ERR_NONE)
	{
		if (err == CR_ERR_EOLF){
			str_cpy(CR_ERROR_STR, CR_STR_ERR_EOLF);
		}else{
		if (err == CR_ERR_PARAM){
			str_cpy(CR_ERROR_STR, CR_STR_ERR_PARAM);
		}else{
		if (err == CR_ERR_BADFILE){
			str_cpy(CR_ERROR_STR, CR_STR_ERR_BADFILE);
		}else{
		if (err == CR_ERR_INTERNAL){
			str_cpy(CR_ERROR_STR, CR_STR_ERR_INTERNAL);
		}else{
		if (err == CR_ERR_CRC){
			str_cpy(CR_ERROR_STR, CR_STR_ERR_CRC);
		}else{
		if (err == CR_ERR_ERRNO){
			str_cpy(CR_ERROR_STR, CR_STR_ERR_ERRNO);
		}else{
		if (err == CR_ERR_STREAM){
			str_cpy(CR_ERROR_STR, CR_STR_ERR_STREAM);
		}else{
		if (err == CR_ERR_DATA){
			str_cpy(CR_ERROR_STR, CR_STR_ERR_DATA);
		}else{
		if (err == CR_ERR_MEM){
			str_cpy(CR_ERROR_STR, CR_STR_ERR_MEM);
		}else{
		if (err == CR_ERR_BUF){
			str_cpy(CR_ERROR_STR, CR_STR_ERR_BUF);
		}else{
		if (err == CR_ERR_VERSION){
			str_cpy(CR_ERROR_STR, CR_STR_ERR_VERSION);
		}else{
			str_cpy(CR_ERROR_STR, CR_STR_ERR_UNKNOWN);
		}}}}}}}}}}}
		str_cat(CR_ERROR_STR, CR_STR_ERR_RESPREF);
		str_cat(CR_ERROR_STR, FileName);
IFDEF ShowErrorToScreen;
		error(CR_ERROR_STR);
ENDIF;
IFDEF PrintErrorToAcklog;
		str_cat(CR_ERROR_STR, "\n");
		diag("\ncustres.wdl: CR_ShowErr -> ");
		diag(CR_ERROR_STR);
ENDIF;
IFDEF AbortOnErrorInGSR;
		sys_exit(CR_STR_ERR_ABORT);
IFELSE;
		if (AbortOnError){
			sys_exit(CR_STR_ERR_ABORT);
		}
ENDIF;
	}
}

/*
**	Helper functions to deal with directly imported from CUSTRES.DLL
**	You may use these functions instead of 'low level' one
*/

//open custom resource
function CR_Open(FileName, Password, AbortOnError)
{
	var err = CR_OK;
	Diag("\ncustres.wdl: CR_Open ");
IFDEF UseFileNameStr;
	str_cpy(CR_TEMP_FNAME, CR_FileName_str);
IFELSE;
	str_cpy(CR_TEMP_FNAME, FileName);
ENDIF;
IFDEF UsePasswordStr;
	str_cpy(CR_TEMP_PASS, CR_Password_str);
IFELSE;
	str_cpy(CR_TEMP_PASS, Password);
ENDIF;
	Diag(CR_TEMP_FNAME);
	err = CR_open_(CR_TEMP_FNAME, CR_TEMP_PASS);
	CR_ShowErr(err, CR_TEMP_FNAME, AbortOnError);
	return(err);
}
//close custom resource
function CR_Close(FileName, AbortOnError)
{
	var err = CR_OK;
	Diag("\ncustres.wdl: CR_Close ");
IFDEF UseFileNameStr;
	str_cpy(CR_TEMP_FNAME, CR_FileName_str);
IFELSE;
	str_cpy(CR_TEMP_FNAME, FileName);
ENDIF;
	Diag(CR_TEMP_FNAME);
	err = CR_close_(CR_TEMP_FNAME);
	CR_ShowErr(err, CR_TEMP_FNAME, AbortOnError);
	return(err);
}
/*
**	Resource loading functions
*/
//load all resources
function CR_LoadAll(FileName, AbortOnError)
{
	var err = CR_OK;
	Diag("\ncustres.wdl: CR_LoadAll ");
IFDEF UseFileNameStr;
	str_cpy(CR_TEMP_FNAME, CR_FileName_str);
IFELSE;
	str_cpy(CR_TEMP_FNAME, FileName);
ENDIF;
	Diag(CR_TEMP_FNAME);
	err = CR_addall_(CR_TEMP_FNAME);
	CR_ShowErr(err, CR_TEMP_FNAME, AbortOnError);
	return(err);
}
//load specified resources
function CR_LoadFiles(FileName, FileList, AbortOnError)
{
	var err = CR_OK;
	Diag("\ncustres.wdl: CR_LoadFiles ");
IFDEF UseFileNameStr;
	str_cpy(CR_TEMP_FNAME, CR_FileName_str);
IFELSE;
	str_cpy(CR_TEMP_FNAME, FileName);
ENDIF;
	Diag(CR_TEMP_FNAME);
	err = CR_add_(CR_TEMP_FNAME, FileList);
	CR_ShowErr(err, CR_TEMP_FNAME, AbortOnError);
	return(err);
}
//load item by number
function CR_LoadFileNum(FileName, ItemNumber, AbortOnError)
{
	var err = CR_OK;
	Diag("\ncustres.wdl: CR_LoadFileNum ");
IFDEF UseFileNameStr;
	str_cpy(CR_TEMP_FNAME, CR_FileName_str);
IFELSE;
	str_cpy(CR_TEMP_FNAME, FileName);
ENDIF;
	Diag(CR_TEMP_FNAME);
	err = CR_addn_(CR_TEMP_FNAME, ItemNumber);
	CR_ShowErr(err, CR_TEMP_FNAME, AbortOnError);
	return(err);
}
//load one item by name
function CR_LoadFile(FileName, ItemName, AbortOnError)
{
	var err = CR_OK;
	Diag("\ncustres.wdl: CR_LoadFile ");
IFDEF UseFileNameStr;
	str_cpy(CR_TEMP_FNAME, CR_FileName_str);
IFELSE;
	str_cpy(CR_TEMP_FNAME, FileName);
ENDIF;
	Diag(CR_TEMP_FNAME);
	err = CR_addone_(CR_TEMP_FNAME, ItemName);
	CR_ShowErr(err, CR_TEMP_FNAME, AbortOnError);
	return(err);
}
/*
**	Resource extracting functions
*/
//extract all resources
function CR_ExtractAll(FileName, FilePath, AbortOnError)
{
	var err = CR_OK;
	Diag("\ncustres.wdl: CR_ExtractAll ");
IFDEF UseFileNameStr;
	str_cpy(CR_TEMP_FNAME, CR_FileName_str);
IFELSE;
	str_cpy(CR_TEMP_FNAME, FileName);
ENDIF;
	Diag(CR_TEMP_FNAME);
	err = CR_extractall_(CR_TEMP_FNAME, FilePath);
	CR_ShowErr(err, CR_TEMP_FNAME, AbortOnError);
	return(err);
}
//extract specified resources
function CR_ExtractFiles(FileName, FilePath, FileList, AbortOnError)
{
	var err = CR_OK;
	Diag("\ncustres.wdl: CR_ExtractFiles ");
IFDEF UseFileNameStr;
	str_cpy(CR_TEMP_FNAME, CR_FileName_str);
IFELSE;
	str_cpy(CR_TEMP_FNAME, FileName);
ENDIF;
	Diag(CR_TEMP_FNAME);
	err = CR_extract_(CR_TEMP_FNAME, FilePath, FileList);
	CR_ShowErr(err, CR_TEMP_FNAME, AbortOnError);
	return(err);
}
//extract one resources by number
function CR_ExtractFileNum(FileName, FilePath, ItemNumber, AbortOnError)
{
	var err = CR_OK;
	Diag("\ncustres.wdl: CR_ExtractFileNum ");
IFDEF UseFileNameStr;
	str_cpy(CR_TEMP_FNAME, CR_FileName_str);
IFELSE;
	str_cpy(CR_TEMP_FNAME, FileName);
ENDIF;
	Diag(CR_TEMP_FNAME);
	err = CR_extractn_(CR_TEMP_FNAME, FilePath, ItemNumber);
	CR_ShowErr(err, CR_TEMP_FNAME, AbortOnError);
	return(err);
}
//extract one resources by name
function CR_ExtractFile(FileName, FilePath, ItemName, AbortOnError)
{
	var err = CR_OK;
	Diag("\ncustres.wdl: CR_ExtractFile ");
IFDEF UseFileNameStr;
	str_cpy(CR_TEMP_FNAME, CR_FileName_str);
IFELSE;
	str_cpy(CR_TEMP_FNAME, FileName);
ENDIF;
	Diag(CR_TEMP_FNAME);
	err = CR_extractone_(CR_TEMP_FNAME, FilePath, ItemName);
	CR_ShowErr(err, CR_TEMP_FNAME, AbortOnError);
	return(err);
}
/*
**	Resource unloading functions
*/
//unload all resources
function CR_UnLoadAll(FileName, AbortOnError)
{
	var err = CR_OK;
	Diag("\ncustres.wdl: CR_UnLoadAll ");
IFDEF UseFileNameStr;
	str_cpy(CR_TEMP_FNAME, CR_FileName_str);
IFELSE;
	str_cpy(CR_TEMP_FNAME, FileName);
ENDIF;
	Diag(CR_TEMP_FNAME);
	err = CR_delall_(CR_TEMP_FNAME);
	CR_ShowErr(err, CR_TEMP_FNAME, AbortOnError);
	return(err);
}
//unload specified resources
function CR_UnLoadFiles(FileName, FileList, AbortOnError)
{
	var err = CR_OK;
	Diag("\ncustres.wdl: CR_UnLoadFiles ");
IFDEF UseFileNameStr;
	str_cpy(CR_TEMP_FNAME, CR_FileName_str);
IFELSE;
	str_cpy(CR_TEMP_FNAME, FileName);
ENDIF;
	Diag(CR_TEMP_FNAME);
	err = CR_del_(CR_TEMP_FNAME, FileList);
	CR_ShowErr(err, CR_TEMP_FNAME, AbortOnError);
	return(err);
}
//unload one resource by number
function CR_UnLoadFileNum(FileName, ItemNumber, AbortOnError)
{
	var err = CR_OK;
	Diag("\ncustres.wdl: CR_UnLoadFileNum ");
IFDEF UseFileNameStr;
	str_cpy(CR_TEMP_FNAME, CR_FileName_str);
IFELSE;
	str_cpy(CR_TEMP_FNAME, FileName);
ENDIF;
	Diag(CR_TEMP_FNAME);
	err = CR_deln_(CR_TEMP_FNAME, ItemNumber);
	CR_ShowErr(err, CR_TEMP_FNAME, AbortOnError);
	return(err);
}
//unload one resource by name
function CR_UnLoadFile(FileName, ItemName, AbortOnError)
{
	var err = CR_OK;
	Diag("\ncustres.wdl: CR_UnLoadFile ");
IFDEF UseFileNameStr;
	str_cpy(CR_TEMP_FNAME, CR_FileName_str);
IFELSE;
	str_cpy(CR_TEMP_FNAME, FileName);
ENDIF;
	Diag(CR_TEMP_FNAME);
	err = CR_delone_(CR_TEMP_FNAME, ItemName);
	CR_ShowErr(err, CR_TEMP_FNAME, AbortOnError);
	return(err);
}
/*
**	Resource informating functions
*/
//get number of files in the specified resource
function CR_FileCount(FileName, AbortOnError)
{
	var err = CR_OK;
	Diag("\ncustres.wdl: CR_FileCount ");
IFDEF UseFileNameStr;
	str_cpy(CR_TEMP_FNAME, CR_FileName_str);
IFELSE;
	str_cpy(CR_TEMP_FNAME, FileName);
ENDIF;
	Diag(CR_TEMP_FNAME);
	err = CR_getfcount_(CR_TEMP_FNAME);
	CR_ShowErr(err, CR_TEMP_FNAME, AbortOnError);
	return(err);
}
//get list of files for the resource
function CR_FileList(FileName, FileList, AbortOnError)
{
	var err = CR_OK;
	Diag("\ncustres.wdl: CR_FileList ");
IFDEF UseFileNameStr;
	str_cpy(CR_TEMP_FNAME, CR_FileName_str);
IFELSE;
	str_cpy(CR_TEMP_FNAME, FileName);
ENDIF;
	Diag(CR_TEMP_FNAME);
	err = CR_getflist_(CR_TEMP_FNAME, FileList);
	CR_ShowErr(err, CR_TEMP_FNAME, AbortOnError);
	return(err);
}
//check for item existance in the resource
function CR_ItemExists(FileName, ItemName, AbortOnError)
{
	var err = CR_OK;
	Diag("\ncustres.wdl: CR_ItemExists ");
IFDEF UseFileNameStr;
	str_cpy(CR_TEMP_FNAME, CR_FileName_str);
IFELSE;
	str_cpy(CR_TEMP_FNAME, FileName);
ENDIF;
	Diag(CR_TEMP_FNAME);
	err = CR_itemexists_(CR_TEMP_FNAME, ItemName);
	CR_ShowErr(err, CR_TEMP_FNAME, AbortOnError);
	return(err==CR_OK);
}

IFDEF CR_QuickMode;
/*
** simplified set of functions to use with ONE package and predefined password
*/
//open custom resource
function CR_OpenPack()
{
	var err = CR_OK;
	Diag("\ncustres.wdl: CR_OpenPack");
	err = CR_Open("", "", On);
	return(err);
}
//close custom resource
function CR_ClosePack()
{
	var err = CR_OK;
	Diag("\ncustres.wdl: CR_ClosePack");
	err = CR_Close("", On);
	return(err);
}
//load all resources
function CR_LoadPack()
{
	var err = CR_OK;
	Diag("\ncustres.wdl: CR_LoadPack");
	err = CR_LoadAll("", On);
	return(err);
}
//unload all resources
function CR_UnLoadPack()
{
	var err = CR_OK;
	Diag("\ncustres.wdl: CR_UnLoadPack");
	err = CR_UnLoadAll("", On);
	return(err);
}
//extract specified resources
function CR_ExtractPack(FilePath, FileList)
{
	var err = CR_OK;
	Diag("\ncustres.wdl: CR_ExtractPack");
	err = CR_ExtractFiles("", FilePath, FileList, On);
	return(err);
}
//get package contents
function CR_ListPack(FileList)
{
	var err = CR_OK;
	Diag("\ncustres.wdl: CR_ListPack");
	err = CR_FileList("", FileList, On);
	return(err);
}
//get file count
function CR_CountPack()
{
	var err = CR_OK;
	Diag("\ncustres.wdl: CR_CountPack");
	err = CR_FileCount("", On);
	return(err);
}
//check item existence
function CR_ExistsInPack(ItemName)
{
	var err = CR_OK;
	Diag("\ncustres.wdl: CR_ExistsInPack");
	err = CR_ItemExists("", ItemName, On);
	return(err);
}
ENDIF;

//write to log startup info
function CR_Log_startup
{
	Diag("\n\ncustres.wdl: Options in custres.wdl are");
IFDEF CR_QuickMode;
	Diag("\n - Quick Mode selected;");
ENDIF;
IFDEF UseFileNameStr;
	Diag("\n - work with one CR Package;");
ENDIF;
IFDEF UsePasswordStr;
	Diag("\n - use one password for all CR Packages;");
ENDIF;
IFDEF ShowErrorToScreen;
	Diag("\n - show errors with error() function;");
IFELSE;
	Diag("\n - don't show errors with error() function;");
ENDIF;
IFDEF PrintErrorToAcklog;
	Diag("\n - write errors to acklog.txt;");
IFELSE;
	Diag("\n - don't write errors to acklog.txt;");
ENDIF;
IFDEF AbortOnErrorInGSR;
	Diag("\n - exit on errors in custom resources anyway;");
IFELSE;
	Diag("\n - exit on errors in custom resources when specified only;");
ENDIF;
	Diag("\n\n");
}
endif;	//custres.wdl